import aw_client
from datetime import datetime, timedelta
import socket
import netifaces
import json
import requests

# Initialize the ActivityWatch client
client = aw_client.ActivityWatchClient("my-client")

# Get the current time and calculate the time for the last 60 minutes (using UTC for consistency with ActivityWatch)
now = datetime.now()
time_60_minutes_ago = now - timedelta(minutes=60)

# Truncate to the start of the hour
time_60_minutes_ago = datetime(time_60_minutes_ago.year, time_60_minutes_ago.month, time_60_minutes_ago.day, time_60_minutes_ago.hour)

# Get the buckets for window-related activities (this will include apps you've used and afk time)
buckets = client.get_buckets()

# Filter buckets for those related to windows, afk (AFK/away-from-keyboard events), and web (browser activity)
window_buckets = [bucket for bucket in buckets.values() if 'window' in bucket['client']]
afk_buckets = [bucket for bucket in buckets.values() if 'afk' in bucket['id']]
web_buckets = [bucket for bucket in buckets.values() if 'web' in bucket['id']]

# Collect all events from the window, afk, and web buckets
all_events = []

bucket_event_map = {}  # Dictionary to map bucket ids to their events
for bucket in window_buckets + afk_buckets + web_buckets:
    # Use the start parameter to filter events from the last 60 minutes
    events = client.get_events(bucket['id'], start=time_60_minutes_ago)
    bucket_event_map[bucket['id']] = events
    all_events.extend(events)

# Function to convert UTC timestamp to local time (computer's local time)
def format_time(timestamp):
    if isinstance(timestamp, datetime):
        utc_time = timestamp
    else:
        utc_time = datetime.fromisoformat(timestamp.replace("Z", "+00:00"))

    # Convert from UTC to local time using the system's time zone
    local_time = utc_time.astimezone()  # Automatically convert to the local time zone of the computer
    
    return local_time

# Get the machine's hostname and non-loopback IP address
def get_machine_info():
    hostname = socket.gethostname()
    ip_address = None
    
    # Get all network interfaces and their IP addresses
    for interface in netifaces.interfaces():
        addresses = netifaces.ifaddresses(interface)
        if netifaces.AF_INET in addresses:
            for link in addresses[netifaces.AF_INET]:
                ip = link['addr']
                # Skip loopback addresses
                if not ip.startswith("127."):
                    ip_address = ip
                    break
        if ip_address:
            break
    
    if ip_address is None:
        ip_address = 'Unable to get IP address'
    
    return hostname, ip_address

# Process and display each event with app name, window title, and activity periods or afk periods
app_activity_periods = {}
afk_periods = []
web_activity = []

# Process events based on their bucket type (window, afk, web)
for bucket_id, events in bucket_event_map.items():
    if bucket_id in [bucket['id'] for bucket in web_buckets]:  # Handle web activity
        for event in events:
            url = event['data'].get('url', 'Unknown URL')
            title = event['data'].get('title', 'Unknown Title')
            start_time = format_time(event['timestamp'])
            if 'duration' in event and isinstance(event['duration'], timedelta):
                duration = event['duration'].total_seconds() / 60  # Duration in minutes
                start_time = datetime(start_time.year, start_time.month, start_time.day, start_time.hour)

                web_activity.append({
                    'url': url,
                    'title': title,
                    'start_time': start_time.strftime('%Y%m%d %H:%M:%S'),
                    'duration': duration
                })

    elif bucket_id in [bucket['id'] for bucket in afk_buckets]:  # Handle afk activity
        for event in events:
            status = event['data'].get('status')
            start_time = format_time(event['timestamp'])

            if 'duration' in event and isinstance(event['duration'], timedelta):
                duration = event['duration'].total_seconds() / 60  # Duration in minutes
                start_time = datetime(start_time.year, start_time.month, start_time.day, start_time.hour)

                afk_periods.append({
                    'status': status,
                    'start_time': start_time.strftime('%Y%m%d %H:%M:%S'),
                    'duration': duration
                })

    else:  # Handle app activity (window activity)
        for event in events:
            app = event['data'].get('app')
            title = event['data'].get('title', 'Unknown title')  # Get window title, default to 'Unknown title' if not found
            if app:
                start_time = format_time(event['timestamp'])

                if app not in app_activity_periods:
                    app_activity_periods[app] = []

                if 'duration' in event and isinstance(event['duration'], timedelta):
                    duration = event['duration'].total_seconds() / 60  # Duration in minutes
                    start_time = datetime(start_time.year, start_time.month, start_time.day, start_time.hour)

                    app_activity_periods[app].append({
                        'title': title,
                        'start_time': start_time.strftime('%Y%m%d %H:%M:%S'),
                        'duration': duration
                    })

# Collect the data for the JSON structure
hostname, ip_address = get_machine_info()
n = 0
data = {}
data['hostname'] = hostname
data['ip_address'] = ip_address

for program in app_activity_periods:
    duration  = 0.0
    start_time = 'xxx'

    for event in app_activity_periods[program]:
        if (duration > 0 and start_time != event['start_time']):
            n += 1
            data['name_'+str(n)] = program
            data['start_'+str(n)] = start_time
            data['duration_'+str(n)] = round(duration, 2)
            duration = 0.0

        start_time = event['start_time']
        duration += event['duration']

    n += 1
    data['name_'+str(n)] = program
    data['start_'+str(n)] = start_time
    data['duration_'+str(n)] = round(duration, 2)
    duration = 0.0

actDuration = 0.0
actStart    = 'xxx'
inaDuration = 0.0
inaStart    = 'xxx'

for event in afk_periods:
    if (event['status'] == 'afk'):
        if (actDuration > 0 and actStart != event['start_time']):
            n += 1
            data['name_'+str(n)] = 'afk-Active'
            data['start_'+str(n)] = actStart
            data['duration_'+str(n)] = round(actDuration, 2)
            actDuration = 0.0

        actStart = event['start_time']
        actDuration += event['duration']

    if (event['status'] == 'not-afk'):
        if (inaDuration > 0 and inaStart != event['start_time']):
            n += 1
            data['name_'+str(n)] = 'afk-Inactive'
            data['start_'+str(n)] = inaStart
            data['duration_'+str(n)] = round(inaDuration, 2)
            inaDuration = 0.0

        inaStart = event['start_time']
        inaDuration += event['duration']

if (actDuration > 0.0):
    n += 1
    data['name_'+str(n)] = 'afk-Active'
    data['start_'+str(n)] = actStart
    data['duration_'+str(n)] = round(actDuration, 2)

if (inaDuration > 0.0):
    n += 1
    data['name_'+str(n)] = 'afk-Inactive'
    data['start_'+str(n)] = inaStart
    data['duration_'+str(n)] = round(inaDuration, 2)

data['n'] = n

url = 'https://panel.qinaya.co/api/mobile.asp?action=monitor'
response = requests.post(url, data=data)

print(response)


def main():
    print("If you're seeing this Qinaya rules'.")
